@@ -6,17 +6,14 @@ extern crate test;
 use {
     rand::seq::SliceRandom,
     raptorq::{Decoder, Encoder},
-    solana_ledger::{
-        entry::{create_ticks, Entry},
-        shred::{
-            max_entries_per_n_shred, max_ticks_per_n_shreds, ProcessShredsStats, Shred, Shredder,
-            MAX_DATA_SHREDS_PER_FEC_BLOCK, SHRED_PAYLOAD_SIZE, SIZE_OF_CODING_SHRED_HEADERS,
-            SIZE_OF_DATA_SHRED_PAYLOAD,
-        },
+    solana_entry::entry::{create_ticks, Entry},
+    solana_ledger::shred::{
+        max_entries_per_n_shred, max_ticks_per_n_shreds, ProcessShredsStats, Shred, Shredder,
+        MAX_DATA_SHREDS_PER_FEC_BLOCK, SHRED_PAYLOAD_SIZE, SIZE_OF_CODING_SHRED_HEADERS,
+        SIZE_OF_DATA_SHRED_PAYLOAD,
     },
     solana_perf::test_tx,
     solana_sdk::{hash::Hash, signature::Keypair},
-    std::sync::Arc,
     test::Bencher,
 };
 
@@ -24,7 +21,7 @@ fn make_test_entry(txs_per_entry: u64) -> Entry {
     Entry {
         num_hashes: 100_000,
         hash: Hash::default(),
-        transactions: vec![test_tx::test_tx(); txs_per_entry as usize],
+        transactions: vec![test_tx::test_tx().into(); txs_per_entry as usize],
     }
 }
 fn make_large_unchained_entries(txs_per_entry: u64, num_entries: u64) -> Vec<Entry> {
@@ -42,9 +39,10 @@ fn make_shreds(num_shreds: usize) -> Vec<Shred> {
         Some(shred_size),
     );
     let entries = make_large_unchained_entries(txs_per_entry, num_entries);
-    let shredder = Shredder::new(1, 0, Arc::new(Keypair::new()), 0, 0).unwrap();
+    let shredder = Shredder::new(1, 0, 0, 0).unwrap();
     let data_shreds = shredder
         .entries_to_data_shreds(
+            &Keypair::new(),
             &entries,
             true, // is_last_in_slot
             0,    // next_shred_index
@@ -70,21 +68,21 @@ fn make_concatenated_shreds(num_shreds: usize) -> Vec<u8> {
 
 #[bench]
 fn bench_shredder_ticks(bencher: &mut Bencher) {
-    let kp = Arc::new(Keypair::new());
+    let kp = Keypair::new();
     let shred_size = SIZE_OF_DATA_SHRED_PAYLOAD;
     let num_shreds = ((1000 * 1000) + (shred_size - 1)) / shred_size;
     // ~1Mb
     let num_ticks = max_ticks_per_n_shreds(1, Some(SIZE_OF_DATA_SHRED_PAYLOAD)) * num_shreds as u64;
     let entries = create_ticks(num_ticks, 0, Hash::default());
     bencher.iter(|| {
-        let shredder = Shredder::new(1, 0, kp.clone(), 0, 0).unwrap();
-        shredder.entries_to_shreds(&entries, true, 0);
+        let shredder = Shredder::new(1, 0, 0, 0).unwrap();
+        shredder.entries_to_shreds(&kp, &entries, true, 0);
     })
 }
 
 #[bench]
 fn bench_shredder_large_entries(bencher: &mut Bencher) {
-    let kp = Arc::new(Keypair::new());
+    let kp = Keypair::new();
     let shred_size = SIZE_OF_DATA_SHRED_PAYLOAD;
     let num_shreds = ((1000 * 1000) + (shred_size - 1)) / shred_size;
     let txs_per_entry = 128;
@@ -96,21 +94,21 @@ fn bench_shredder_large_entries(bencher: &mut Bencher) {
     let entries = make_large_unchained_entries(txs_per_entry, num_entries);
     // 1Mb
     bencher.iter(|| {
-        let shredder = Shredder::new(1, 0, kp.clone(), 0, 0).unwrap();
-        shredder.entries_to_shreds(&entries, true, 0);
+        let shredder = Shredder::new(1, 0, 0, 0).unwrap();
+        shredder.entries_to_shreds(&kp, &entries, true, 0);
     })
 }
 
 #[bench]
 fn bench_deshredder(bencher: &mut Bencher) {
-    let kp = Arc::new(Keypair::new());
+    let kp = Keypair::new();
     let shred_size = SIZE_OF_DATA_SHRED_PAYLOAD;
     // ~10Mb
     let num_shreds = ((10000 * 1000) + (shred_size - 1)) / shred_size;
     let num_ticks = max_ticks_per_n_shreds(1, Some(shred_size)) * num_shreds as u64;
     let entries = create_ticks(num_ticks, 0, Hash::default());
-    let shredder = Shredder::new(1, 0, kp, 0, 0).unwrap();
-    let data_shreds = shredder.entries_to_shreds(&entries, true, 0).0;
+    let shredder = Shredder::new(1, 0, 0, 0).unwrap();
+    let data_shreds = shredder.entries_to_shreds(&kp, &entries, true, 0).0;
     bencher.iter(|| {
         let raw = &mut Shredder::deshred(&data_shreds).unwrap();
         assert_ne!(raw.len(), 0);
