@@ -4,14 +4,13 @@
 //! to the GPU.
 //!
 
-use crate::sigverify_stage::SigVerifier;
-use solana_perf::cuda_runtime::PinnedVec;
-use solana_perf::packet::Packets;
-use solana_perf::recycler::Recycler;
-use solana_perf::sigverify;
 pub use solana_perf::sigverify::{
     batch_size, ed25519_verify_cpu, ed25519_verify_disabled, init, TxOffset,
 };
+use {
+    crate::sigverify_stage::SigVerifier,
+    solana_perf::{cuda_runtime::PinnedVec, packet::Packets, recycler::Recycler, sigverify},
+};
 
 #[derive(Clone)]
 pub struct TransactionSigVerifier {
