@@ -9,11 +9,11 @@ use {
     crate::sigverify,
     crossbeam_channel::{SendError, Sender as CrossbeamSender},
     solana_measure::measure::Measure,
-    solana_perf::packet::PacketBatch,
+    solana_perf::packet::Packets,
     solana_sdk::timing,
-    solana_streamer::streamer::{self, PacketBatchReceiver, StreamerError},
+    solana_streamer::streamer::{self, PacketReceiver, StreamerError},
     std::{
-        collections::{HashMap, VecDeque},
+        collections::HashMap,
         sync::mpsc::{Receiver, RecvTimeoutError},
         thread::{self, Builder, JoinHandle},
         time::Instant,
@@ -26,7 +26,7 @@ const MAX_SIGVERIFY_BATCH: usize = 10_000;
 #[derive(Error, Debug)]
 pub enum SigVerifyServiceError {
     #[error("send packets batch error")]
-    Send(#[from] SendError<Vec<PacketBatch>>),
+    Send(#[from] SendError<Vec<Packets>>),
 
     #[error("streamer error")]
     Streamer(#[from] StreamerError),
@@ -39,7 +39,7 @@ pub struct SigVerifyStage {
 }
 
 pub trait SigVerifier {
-    fn verify_batches(&self, batches: Vec<PacketBatch>) -> Vec<PacketBatch>;
+    fn verify_batch(&self, batch: Vec<Packets>) -> Vec<Packets>;
 }
 
 #[derive(Default, Clone)]
@@ -49,8 +49,7 @@ pub struct DisabledSigVerifier {}
 struct SigVerifierStats {
     recv_batches_us_hist: histogram::Histogram, // time to call recv_batch
     verify_batches_pp_us_hist: histogram::Histogram, // per-packet time to call verify_batch
-    discard_packets_pp_us_hist: histogram::Histogram, // per-packet time to call verify_batch
-    batches_hist: histogram::Histogram,         // number of packet batches per verify call
+    batches_hist: histogram::Histogram,         // number of Packets structures per verify call
     packets_hist: histogram::Histogram,         // number of packets per verify call
     total_batches: usize,
     total_packets: usize,
@@ -100,28 +99,6 @@ impl SigVerifierStats {
                 self.verify_batches_pp_us_hist.mean().unwrap_or(0),
                 i64
             ),
-            (
-                "discard_packets_pp_us_90pct",
-                self.discard_packets_pp_us_hist
-                    .percentile(90.0)
-                    .unwrap_or(0),
-                i64
-            ),
-            (
-                "discard_packets_pp_us_min",
-                self.discard_packets_pp_us_hist.minimum().unwrap_or(0),
-                i64
-            ),
-            (
-                "discard_packets_pp_us_max",
-                self.discard_packets_pp_us_hist.maximum().unwrap_or(0),
-                i64
-            ),
-            (
-                "discard_packets_pp_us_mean",
-                self.discard_packets_pp_us_hist.mean().unwrap_or(0),
-                i64
-            ),
             (
                 "batches_90pct",
                 self.batches_hist.percentile(90.0).unwrap_or(0),
@@ -145,40 +122,40 @@ impl SigVerifierStats {
 }
 
 impl SigVerifier for DisabledSigVerifier {
-    fn verify_batches(&self, mut batches: Vec<PacketBatch>) -> Vec<PacketBatch> {
-        sigverify::ed25519_verify_disabled(&mut batches);
-        batches
+    fn verify_batch(&self, mut batch: Vec<Packets>) -> Vec<Packets> {
+        sigverify::ed25519_verify_disabled(&mut batch);
+        batch
     }
 }
 
 impl SigVerifyStage {
     #[allow(clippy::new_ret_no_self)]
     pub fn new<T: SigVerifier + 'static + Send + Clone>(
-        packet_receiver: Receiver<PacketBatch>,
-        verified_sender: CrossbeamSender<Vec<PacketBatch>>,
+        packet_receiver: Receiver<Packets>,
+        verified_sender: CrossbeamSender<Vec<Packets>>,
         verifier: T,
     ) -> Self {
         let thread_hdl = Self::verifier_services(packet_receiver, verified_sender, verifier);
         Self { thread_hdl }
     }
 
-    pub fn discard_excess_packets(batches: &mut Vec<PacketBatch>, max_packets: usize) {
+    pub fn discard_excess_packets(batches: &mut Vec<Packets>, max_packets: usize) {
         let mut received_ips = HashMap::new();
         for (batch_index, batch) in batches.iter().enumerate() {
             for (packet_index, packets) in batch.packets.iter().enumerate() {
                 let e = received_ips
                     .entry(packets.meta.addr().ip())
-                    .or_insert_with(VecDeque::new);
-                e.push_back((batch_index, packet_index));
+                    .or_insert_with(Vec::new);
+                e.push((batch_index, packet_index));
             }
         }
         let mut batch_len = 0;
         while batch_len < max_packets {
             for (_ip, indexes) in received_ips.iter_mut() {
                 if !indexes.is_empty() {
-                    indexes.pop_front();
+                    indexes.remove(0);
                     batch_len += 1;
-                    if batch_len >= max_packets {
+                    if batch_len >= MAX_SIGVERIFY_BATCH {
                         break;
                     }
                 }
@@ -192,22 +169,25 @@ impl SigVerifyStage {
     }
 
     fn verifier<T: SigVerifier>(
-        recvr: &PacketBatchReceiver,
-        sendr: &CrossbeamSender<Vec<PacketBatch>>,
+        recvr: &PacketReceiver,
+        sendr: &CrossbeamSender<Vec<Packets>>,
         verifier: &T,
         stats: &mut SigVerifierStats,
     ) -> Result<()> {
-        let (mut batches, len, recv_time) = streamer::recv_batch(recvr)?;
+        let (mut batches, num_packets, recv_duration) = streamer::recv_batch(recvr)?;
 
-        let mut verify_batch_time = Measure::start("sigverify_batch_time");
         let batches_len = batches.len();
-        debug!("@{:?} verifier: verifying: {}", timing::timestamp(), len,);
-        let mut discard_time = Measure::start("sigverify_discard_time");
-        if len > MAX_SIGVERIFY_BATCH {
+        debug!(
+            "@{:?} verifier: verifying: {}",
+            timing::timestamp(),
+            num_packets,
+        );
+        if num_packets > MAX_SIGVERIFY_BATCH {
             Self::discard_excess_packets(&mut batches, MAX_SIGVERIFY_BATCH);
         }
-        discard_time.stop();
-        sendr.send(verifier.verify_batches(batches))?;
+
+        let mut verify_batch_time = Measure::start("sigverify_batch_time");
+        sendr.send(verifier.verify_batch(batches))?;
         verify_batch_time.stop();
 
         debug!(
@@ -215,41 +195,29 @@ impl SigVerifyStage {
             timing::timestamp(),
             batches_len,
             verify_batch_time.as_ms(),
-            len,
-            (len as f32 / verify_batch_time.as_s())
-        );
-
-        datapoint_debug!(
-            "sigverify_stage-total_verify_time",
-            ("num_batches", batches_len, i64),
-            ("num_packets", len, i64),
-            ("verify_time_ms", verify_batch_time.as_ms(), i64),
-            ("recv_time", recv_time, i64),
+            num_packets,
+            (num_packets as f32 / verify_batch_time.as_s())
         );
 
         stats
             .recv_batches_us_hist
-            .increment(recv_time as u64)
+            .increment(recv_duration.as_micros() as u64)
             .unwrap();
         stats
             .verify_batches_pp_us_hist
-            .increment(verify_batch_time.as_us() / (len as u64))
-            .unwrap();
-        stats
-            .discard_packets_pp_us_hist
-            .increment(discard_time.as_us() / (len as u64))
+            .increment(verify_batch_time.as_us() / (num_packets as u64))
             .unwrap();
         stats.batches_hist.increment(batches_len as u64).unwrap();
-        stats.packets_hist.increment(len as u64).unwrap();
+        stats.packets_hist.increment(num_packets as u64).unwrap();
         stats.total_batches += batches_len;
-        stats.total_packets += len;
+        stats.total_packets += num_packets;
 
         Ok(())
     }
 
     fn verifier_service<T: SigVerifier + 'static + Send + Clone>(
-        packet_receiver: PacketBatchReceiver,
-        verified_sender: CrossbeamSender<Vec<PacketBatch>>,
+        packet_receiver: PacketReceiver,
+        verified_sender: CrossbeamSender<Vec<Packets>>,
         verifier: &T,
     ) -> JoinHandle<()> {
         let verifier = verifier.clone();
@@ -284,8 +252,8 @@ impl SigVerifyStage {
     }
 
     fn verifier_services<T: SigVerifier + 'static + Send + Clone>(
-        packet_receiver: PacketBatchReceiver,
-        verified_sender: CrossbeamSender<Vec<PacketBatch>>,
+        packet_receiver: PacketReceiver,
+        verified_sender: CrossbeamSender<Vec<Packets>>,
         verifier: T,
     ) -> JoinHandle<()> {
         Self::verifier_service(packet_receiver, verified_sender, &verifier)
@@ -300,12 +268,11 @@ impl SigVerifyStage {
 mod tests {
     use {super::*, solana_perf::packet::Packet};
 
-    fn count_non_discard(packet_batches: &[PacketBatch]) -> usize {
-        packet_batches
+    fn count_non_discard(packets: &[Packets]) -> usize {
+        packets
             .iter()
-            .map(|batch| {
-                batch
-                    .packets
+            .map(|pp| {
+                pp.packets
                     .iter()
                     .map(|p| if p.meta.discard { 0 } else { 1 })
                     .sum::<usize>()
@@ -316,14 +283,14 @@ mod tests {
     #[test]
     fn test_packet_discard() {
         solana_logger::setup();
-        let mut batch = PacketBatch::default();
-        batch.packets.resize(10, Packet::default());
-        batch.packets[3].meta.addr = [1u16; 8];
-        let mut batches = vec![batch];
+        let mut p = Packets::default();
+        p.packets.resize(10, Packet::default());
+        p.packets[3].meta.addr = [1u16; 8];
+        let mut packets = vec![p];
         let max = 3;
-        SigVerifyStage::discard_excess_packets(&mut batches, max);
-        assert_eq!(count_non_discard(&batches), max);
-        assert!(!batches[0].packets[0].meta.discard);
-        assert!(!batches[0].packets[3].meta.discard);
+        SigVerifyStage::discard_excess_packets(&mut packets, max);
+        assert_eq!(count_non_discard(&packets), max);
+        assert!(!packets[0].packets[0].meta.discard);
+        assert!(!packets[0].packets[3].meta.discard);
     }
 }
