@@ -1,19 +1,21 @@
-use solana_sdk::timing::AtomicInterval;
-use std::{
-    collections::HashMap,
-    io::BufRead,
-    sync::{
-        atomic::{AtomicBool, Ordering},
-        Arc,
-    },
-    thread::{self, sleep, Builder, JoinHandle},
-    time::Duration,
-};
 #[cfg(target_os = "linux")]
 use std::{fs::File, io::BufReader, path::Path};
+use {
+    solana_sdk::timing::AtomicInterval,
+    std::{
+        collections::HashMap,
+        io::BufRead,
+        sync::{
+            atomic::{AtomicBool, Ordering},
+            Arc,
+        },
+        thread::{self, sleep, Builder, JoinHandle},
+        time::Duration,
+    },
+};
 
 const MS_PER_S: u64 = 1_000;
-const SAMPLE_INTERVAL_UDP_MS: u64 = 2 * MS_PER_S;
+const SAMPLE_INTERVAL_UDP_MS: u64 = 60 * MS_PER_S;
 const SAMPLE_INTERVAL_MEM_MS: u64 = MS_PER_S;
 const SLEEP_INTERVAL: Duration = Duration::from_millis(500);
 
@@ -128,7 +130,7 @@ impl SystemMonitorService {
     #[cfg(target_os = "linux")]
     fn report_udp_stats(old_stats: &UdpStats, new_stats: &UdpStats) {
         datapoint_info!(
-            "net-stats-validator",
+            "net-stats",
             (
                 "in_datagrams_delta",
                 new_stats.in_datagrams - old_stats.in_datagrams,
@@ -185,16 +187,22 @@ impl SystemMonitorService {
 
     fn report_mem_stats() {
         if let Ok(info) = sys_info::mem_info() {
+            // stats are returned in kb.
+            const KB: u64 = 1_024;
             datapoint_info!(
                 "memory-stats",
-                ("total", info.total, i64),
+                ("total", info.total * KB, i64),
                 ("swap_total", info.swap_total, i64),
                 (
                     "free_percent",
                     Self::calc_percent(info.free, info.total),
                     f64
                 ),
-                ("used_bytes", info.total.saturating_sub(info.avail), i64),
+                (
+                    "used_bytes",
+                    info.total.saturating_sub(info.avail) * KB,
+                    i64
+                ),
                 (
                     "avail_percent",
                     Self::calc_percent(info.avail, info.total),
@@ -229,7 +237,7 @@ impl SystemMonitorService {
                 break;
             }
 
-            if udp_timer.should_update(SAMPLE_INTERVAL_UDP_MS) && report_os_network_stats {
+            if report_os_network_stats && udp_timer.should_update(SAMPLE_INTERVAL_UDP_MS) {
                 SystemMonitorService::process_udp_stats(&mut udp_stats);
             }
 
