@@ -1,6 +1,7 @@
 use {
     crate::{cluster_info_vote_listener::VerifiedLabelVotePacketsReceiver, result::Result},
-    solana_perf::packet::PacketBatch,
+    crossbeam_channel::Select,
+    solana_perf::packet::Packets,
     solana_runtime::bank::Bank,
     solana_sdk::{
         account::from_account, clock::Slot, hash::Hash, pubkey::Pubkey, signature::Signature,
@@ -19,7 +20,7 @@ const MAX_VOTES_PER_VALIDATOR: usize = 1000;
 pub struct VerifiedVoteMetadata {
     pub vote_account_key: Pubkey,
     pub vote: Vote,
-    pub packet_batch: PacketBatch,
+    pub packet: Packets,
     pub signature: Signature,
 }
 
@@ -69,7 +70,7 @@ impl<'a> ValidatorGossipVotesIterator<'a> {
 ///
 /// Iterator is done after iterating through all vote accounts
 impl<'a> Iterator for ValidatorGossipVotesIterator<'a> {
-    type Item = Vec<PacketBatch>;
+    type Item = Vec<Packets>;
 
     fn next(&mut self) -> Option<Self::Item> {
         // TODO: Maybe prioritize by stake weight
@@ -115,7 +116,7 @@ impl<'a> Iterator for ValidatorGossipVotesIterator<'a> {
                                             None
                                         }
                                     })
-                                    .collect::<Vec<PacketBatch>>()
+                                    .collect::<Vec<Packets>>()
                             })
                         })
                 });
@@ -129,7 +130,7 @@ impl<'a> Iterator for ValidatorGossipVotesIterator<'a> {
     }
 }
 
-pub type SingleValidatorVotes = BTreeMap<(Slot, Hash), (PacketBatch, Signature)>;
+pub type SingleValidatorVotes = BTreeMap<(Slot, Hash), (Packets, Signature)>;
 
 #[derive(Default)]
 pub struct VerifiedVotePackets(HashMap<Pubkey, SingleValidatorVotes>);
@@ -140,16 +141,16 @@ impl VerifiedVotePackets {
         vote_packets_receiver: &VerifiedLabelVotePacketsReceiver,
         would_be_leader: bool,
     ) -> Result<()> {
-        const RECV_TIMEOUT: Duration = Duration::from_millis(200);
-        let vote_packets = vote_packets_receiver.recv_timeout(RECV_TIMEOUT)?;
-        let vote_packets = std::iter::once(vote_packets).chain(vote_packets_receiver.try_iter());
-        for gossip_votes in vote_packets {
+        let mut sel = Select::new();
+        sel.recv(vote_packets_receiver);
+        let _ = sel.ready_timeout(Duration::from_millis(200))?;
+        for gossip_votes in vote_packets_receiver.try_iter() {
             if would_be_leader {
                 for verfied_vote_metadata in gossip_votes {
                     let VerifiedVoteMetadata {
                         vote_account_key,
                         vote,
-                        packet_batch,
+                        packet,
                         signature,
                     } = verfied_vote_metadata;
                     if vote.slots.is_empty() {
@@ -160,7 +161,7 @@ impl VerifiedVotePackets {
                     let hash = vote.hash;
 
                     let validator_votes = self.0.entry(vote_account_key).or_default();
-                    validator_votes.insert((*slot, hash), (packet_batch, signature));
+                    validator_votes.insert((*slot, hash), (packet, signature));
 
                     if validator_votes.len() > MAX_VOTES_PER_VALIDATOR {
                         let smallest_key = validator_votes.keys().next().cloned().unwrap();
@@ -177,7 +178,7 @@ impl VerifiedVotePackets {
 mod tests {
     use {
         super::*,
-        crate::{consensus::test::VoteSimulator, result::Error},
+        crate::{result::Error, vote_simulator::VoteSimulator},
         crossbeam_channel::unbounded,
         solana_perf::packet::Packet,
         solana_sdk::slot_hashes::MAX_ENTRIES,
@@ -198,7 +199,7 @@ mod tests {
         s.send(vec![VerifiedVoteMetadata {
             vote_account_key,
             vote: vote.clone(),
-            packet_batch: PacketBatch::default(),
+            packet: Packets::default(),
             signature: Signature::new(&[1u8; 64]),
         }])
         .unwrap();
@@ -218,7 +219,7 @@ mod tests {
         s.send(vec![VerifiedVoteMetadata {
             vote_account_key,
             vote,
-            packet_batch: PacketBatch::default(),
+            packet: Packets::default(),
             signature: Signature::new(&[1u8; 64]),
         }])
         .unwrap();
@@ -240,7 +241,7 @@ mod tests {
         s.send(vec![VerifiedVoteMetadata {
             vote_account_key,
             vote,
-            packet_batch: PacketBatch::default(),
+            packet: Packets::default(),
             signature: Signature::new(&[1u8; 64]),
         }])
         .unwrap();
@@ -263,7 +264,7 @@ mod tests {
         s.send(vec![VerifiedVoteMetadata {
             vote_account_key,
             vote,
-            packet_batch: PacketBatch::default(),
+            packet: Packets::default(),
             signature: Signature::new(&[2u8; 64]),
         }])
         .unwrap();
@@ -282,7 +283,7 @@ mod tests {
         // No new messages, should time out
         assert_matches!(
             verified_vote_packets.receive_and_process_vote_packets(&r, true),
-            Err(Error::CrossbeamRecvTimeout(_))
+            Err(Error::ReadyTimeout)
         );
     }
 
@@ -302,7 +303,7 @@ mod tests {
             s.send(vec![VerifiedVoteMetadata {
                 vote_account_key,
                 vote,
-                packet_batch: PacketBatch::default(),
+                packet: Packets::default(),
                 signature: Signature::new(&[1u8; 64]),
             }])
             .unwrap();
@@ -339,7 +340,7 @@ mod tests {
             s.send(vec![VerifiedVoteMetadata {
                 vote_account_key,
                 vote,
-                packet_batch: PacketBatch::default(),
+                packet: Packets::default(),
                 signature: Signature::new_unique(),
             }])
             .unwrap();
@@ -393,7 +394,7 @@ mod tests {
                 s.send(vec![VerifiedVoteMetadata {
                     vote_account_key,
                     vote,
-                    packet_batch: PacketBatch::new(vec![Packet::default(); num_packets]),
+                    packet: Packets::new(vec![Packet::default(); num_packets]),
                     signature: Signature::new_unique(),
                 }])
                 .unwrap();
@@ -426,7 +427,7 @@ mod tests {
         // Get and verify batches
         let num_expected_batches = 2;
         for _ in 0..num_expected_batches {
-            let validator_batch: Vec<PacketBatch> = gossip_votes_iterator.next().unwrap();
+            let validator_batch: Vec<Packets> = gossip_votes_iterator.next().unwrap();
             assert_eq!(validator_batch.len(), slot_hashes.slot_hashes().len());
             let expected_len = validator_batch[0].packets.len();
             assert!(validator_batch
@@ -460,7 +461,7 @@ mod tests {
         s.send(vec![VerifiedVoteMetadata {
             vote_account_key,
             vote,
-            packet_batch: PacketBatch::default(),
+            packet: Packets::default(),
             signature: Signature::new_unique(),
         }])
         .unwrap();
