@@ -3,11 +3,11 @@
 //!
 use {
     crate::{
+        ancestor_hashes_service::AncestorHashesReplayUpdateReceiver,
         cluster_info_vote_listener::VerifiedVoteReceiver,
-        cluster_slots::ClusterSlots,
         completed_data_sets_service::CompletedDataSetsSender,
         repair_response,
-        repair_service::{OutstandingRepairs, RepairInfo, RepairService},
+        repair_service::{OutstandingShredRepairs, RepairInfo, RepairService},
         result::{Error, Result},
     },
     crossbeam_channel::{
@@ -22,7 +22,7 @@ use {
     },
     solana_measure::measure::Measure,
     solana_metrics::{inc_new_counter_debug, inc_new_counter_error},
-    solana_perf::packet::{Packet, PacketBatch},
+    solana_perf::packet::{Packet, Packets},
     solana_rayon_threadlimit::get_thread_count,
     solana_runtime::{bank::Bank, bank_forks::BankForks},
     solana_sdk::{clock::Slot, packet::PACKET_DATA_SIZE, pubkey::Pubkey},
@@ -124,7 +124,7 @@ impl ReceiveWindowStats {
         const MAX_NUM_ADDRS: usize = 5;
         const SUBMIT_CADENCE: Duration = Duration::from_secs(2);
         let elapsed = self.since.as_ref().map(Instant::elapsed);
-        if elapsed.map(|e| e < SUBMIT_CADENCE).unwrap_or_default() {
+        if elapsed.unwrap_or(Duration::MAX) < SUBMIT_CADENCE {
             return;
         }
         datapoint_info!(
@@ -163,7 +163,10 @@ impl ReceiveWindowStats {
 fn verify_shred_slot(shred: &Shred, root: u64) -> bool {
     match shred.shred_type() {
         // Only data shreds have parent information
-        ShredType::Data => blockstore::verify_shred_slots(shred.slot(), shred.parent(), root),
+        ShredType::Data => match shred.parent() {
+            Ok(parent) => blockstore::verify_shred_slots(shred.slot(), parent, root),
+            Err(_) => false,
+        },
         // Filter out outdated coding shreds
         ShredType::Code => shred.slot() >= root,
     }
@@ -244,18 +247,21 @@ fn run_check_duplicate(
 }
 
 fn verify_repair(
-    outstanding_requests: &mut OutstandingRepairs,
+    outstanding_requests: &mut OutstandingShredRepairs,
     shred: &Shred,
     repair_meta: &Option<RepairMeta>,
 ) -> bool {
     repair_meta
         .as_ref()
         .map(|repair_meta| {
-            outstanding_requests.register_response(
-                repair_meta.nonce,
-                shred,
-                solana_sdk::timing::timestamp(),
-            )
+            outstanding_requests
+                .register_response(
+                    repair_meta.nonce,
+                    shred,
+                    solana_sdk::timing::timestamp(),
+                    |_| (),
+                )
+                .is_some()
         })
         .unwrap_or(true)
 }
@@ -263,7 +269,7 @@ fn verify_repair(
 fn prune_shreds_invalid_repair(
     shreds: &mut Vec<Shred>,
     repair_infos: &mut Vec<Option<RepairMeta>>,
-    outstanding_requests: &RwLock<OutstandingRepairs>,
+    outstanding_requests: &RwLock<OutstandingShredRepairs>,
 ) {
     assert_eq!(shreds.len(), repair_infos.len());
     let mut i = 0;
@@ -296,7 +302,7 @@ fn run_insert<F>(
     ws_metrics: &mut WindowServiceMetrics,
     completed_data_sets_sender: &CompletedDataSetsSender,
     retransmit_sender: &Sender<Vec<Shred>>,
-    outstanding_requests: &RwLock<OutstandingRepairs>,
+    outstanding_requests: &RwLock<OutstandingShredRepairs>,
 ) -> Result<()>
 where
     F: Fn(Shred),
@@ -347,7 +353,7 @@ fn recv_window<F>(
     blockstore: &Blockstore,
     bank_forks: &RwLock<BankForks>,
     insert_shred_sender: &CrossbeamSender<(Vec<Shred>, Vec<Option<RepairMeta>>)>,
-    verified_receiver: &CrossbeamReceiver<Vec<PacketBatch>>,
+    verified_receiver: &CrossbeamReceiver<Vec<Packets>>,
     retransmit_sender: &Sender<Vec<Shred>>,
     shred_filter: F,
     thread_pool: &ThreadPool,
@@ -452,18 +458,17 @@ impl WindowService {
     #[allow(clippy::too_many_arguments)]
     pub(crate) fn new<F>(
         blockstore: Arc<Blockstore>,
-        cluster_info: Arc<ClusterInfo>,
-        verified_receiver: CrossbeamReceiver<Vec<PacketBatch>>,
+        verified_receiver: CrossbeamReceiver<Vec<Packets>>,
         retransmit_sender: Sender<Vec<Shred>>,
         repair_socket: Arc<UdpSocket>,
         exit: Arc<AtomicBool>,
         repair_info: RepairInfo,
         leader_schedule_cache: Arc<LeaderScheduleCache>,
         shred_filter: F,
-        cluster_slots: Arc<ClusterSlots>,
         verified_vote_receiver: VerifiedVoteReceiver,
         completed_data_sets_sender: CompletedDataSetsSender,
         duplicate_slots_sender: DuplicateSlotSender,
+        ancestor_hashes_replay_update_receiver: AncestorHashesReplayUpdateReceiver,
     ) -> WindowService
     where
         F: 'static
@@ -471,26 +476,27 @@ impl WindowService {
             + std::marker::Send
             + std::marker::Sync,
     {
-        let outstanding_requests = Arc::<RwLock<OutstandingRepairs>>::default();
+        let outstanding_requests = Arc::<RwLock<OutstandingShredRepairs>>::default();
 
         let bank_forks = repair_info.bank_forks.clone();
+        let cluster_info = repair_info.cluster_info.clone();
+        let id = cluster_info.id();
 
         let repair_service = RepairService::new(
             blockstore.clone(),
             exit.clone(),
             repair_socket,
-            cluster_info.clone(),
             repair_info,
-            cluster_slots,
             verified_vote_receiver,
             outstanding_requests.clone(),
+            ancestor_hashes_replay_update_receiver,
         );
 
         let (insert_sender, insert_receiver) = unbounded();
         let (duplicate_sender, duplicate_receiver) = unbounded();
 
         let t_check_duplicate = Self::start_check_duplicate_thread(
-            cluster_info.clone(),
+            cluster_info,
             exit.clone(),
             blockstore.clone(),
             duplicate_receiver,
@@ -509,7 +515,7 @@ impl WindowService {
         );
 
         let t_window = Self::start_recv_window_thread(
-            cluster_info.id(),
+            id,
             exit,
             blockstore,
             insert_sender,
@@ -567,7 +573,7 @@ impl WindowService {
         check_duplicate_sender: CrossbeamSender<Shred>,
         completed_data_sets_sender: CompletedDataSetsSender,
         retransmit_sender: Sender<Vec<Shred>>,
-        outstanding_requests: Arc<RwLock<OutstandingRepairs>>,
+        outstanding_requests: Arc<RwLock<OutstandingShredRepairs>>,
     ) -> JoinHandle<()> {
         let mut handle_timeout = || {};
         let handle_error = || {
@@ -623,7 +629,7 @@ impl WindowService {
         exit: Arc<AtomicBool>,
         blockstore: Arc<Blockstore>,
         insert_sender: CrossbeamSender<(Vec<Shred>, Vec<Option<RepairMeta>>)>,
-        verified_receiver: CrossbeamReceiver<Vec<PacketBatch>>,
+        verified_receiver: CrossbeamReceiver<Vec<Packets>>,
         shred_filter: F,
         bank_forks: Arc<RwLock<BankForks>>,
         retransmit_sender: Sender<Vec<Shred>>,
@@ -713,10 +719,10 @@ impl WindowService {
 mod test {
     use {
         super::*,
+        solana_entry::entry::{create_ticks, Entry},
         solana_gossip::contact_info::ContactInfo,
         solana_ledger::{
             blockstore::{make_many_slot_entries, Blockstore},
-            entry::{create_ticks, Entry},
             genesis_utils::create_genesis_config_with_leader,
             get_tmp_ledger_path,
             shred::{DataShredHeader, Shredder},
@@ -734,10 +740,10 @@ mod test {
         entries: &[Entry],
         slot: Slot,
         parent: Slot,
-        keypair: &Arc<Keypair>,
+        keypair: &Keypair,
     ) -> Vec<Shred> {
-        let shredder = Shredder::new(slot, parent, keypair.clone(), 0, 0).unwrap();
-        shredder.entries_to_shreds(entries, true, 0).0
+        let shredder = Shredder::new(slot, parent, 0, 0).unwrap();
+        shredder.entries_to_shreds(keypair, entries, true, 0).0
     }
 
     #[test]
@@ -746,7 +752,7 @@ mod test {
         let blockstore = Arc::new(Blockstore::open(&blockstore_path).unwrap());
         let num_entries = 10;
         let original_entries = create_ticks(num_entries, 0, Hash::default());
-        let mut shreds = local_entries_to_shred(&original_entries, 0, 0, &Arc::new(Keypair::new()));
+        let mut shreds = local_entries_to_shred(&original_entries, 0, 0, &Keypair::new());
         shreds.reverse();
         blockstore
             .insert_shreds(shreds, None, false)
@@ -763,7 +769,7 @@ mod test {
         let me_id = solana_sdk::pubkey::new_rand();
         let leader_keypair = Arc::new(Keypair::new());
         let leader_pubkey = leader_keypair.pubkey();
-        let bank = Arc::new(Bank::new(
+        let bank = Arc::new(Bank::new_for_tests(
             &create_genesis_config_with_leader(100, &leader_pubkey, 10).genesis_config,
         ));
         let cache = Arc::new(LeaderScheduleCache::new_from_bank(&bank));
@@ -779,6 +785,7 @@ mod test {
             0,
             0
         ));
+
         // with the wrong shred_version, shred gets thrown out
         assert!(!should_retransmit_and_persist(
             &shreds[0],
@@ -789,44 +796,26 @@ mod test {
             1
         ));
 
-        // If it's a coding shred, test that slot >= root
-        let (common, coding) = Shredder::new_coding_shred_header(
-            5, // slot
-            5, // index
-            5, // fec_set_index
-            6, // num_data_shreds
-            6, // num_coding_shreds
-            3, // position
-            0, // version
-        );
-        let mut coding_shred =
-            Shred::new_empty_from_header(common, DataShredHeader::default(), coding);
-        Shredder::sign_shred(&leader_keypair, &mut coding_shred);
-        assert!(should_retransmit_and_persist(
-            &coding_shred,
+        // substitute leader_pubkey for me_id so it looks I was the leader
+        // if the shred came back from me, it doesn't continue, whether or not I have a bank
+        assert!(!should_retransmit_and_persist(
+            &shreds[0],
             Some(bank.clone()),
             &cache,
-            &me_id,
+            &leader_pubkey,
             0,
             0
         ));
-        assert!(should_retransmit_and_persist(
-            &coding_shred,
-            Some(bank.clone()),
-            &cache,
-            &me_id,
-            5,
-            0
-        ));
         assert!(!should_retransmit_and_persist(
-            &coding_shred,
-            Some(bank.clone()),
+            &shreds[0],
+            None,
             &cache,
-            &me_id,
-            6,
+            &leader_pubkey,
+            0,
             0
         ));
 
+        // change the shred's slot so leader lookup fails
         // with a Bank and no idea who leader is, shred gets thrown out
         let mut bad_slot_shred = shreds[0].clone();
         bad_slot_shred.set_slot(MINIMUM_SLOTS_PER_EPOCH as u64 * 3);
@@ -851,34 +840,82 @@ mod test {
             0
         ));
 
+        // with an invalid index, shred gets thrown out
+        let mut bad_index_shred = shreds[0].clone();
+        bad_index_shred.common_header.index = (MAX_DATA_SHREDS_PER_SLOT + 1) as u32;
+        assert!(!should_retransmit_and_persist(
+            &bad_index_shred,
+            Some(bank.clone()),
+            &cache,
+            &me_id,
+            0,
+            0
+        ));
+
         // with a shred where shred.slot() == root, shred gets thrown out
-        let slot = MINIMUM_SLOTS_PER_EPOCH as u64 * 3;
-        let shreds = local_entries_to_shred(&[Entry::default()], slot, slot - 1, &leader_keypair);
+        let root = MINIMUM_SLOTS_PER_EPOCH as u64 * 3;
+        let shreds = local_entries_to_shred(&[Entry::default()], root, root - 1, &leader_keypair);
         assert!(!should_retransmit_and_persist(
             &shreds[0],
             Some(bank.clone()),
             &cache,
             &me_id,
-            slot,
+            root,
             0
         ));
 
         // with a shred where shred.parent() < root, shred gets thrown out
-        let slot = MINIMUM_SLOTS_PER_EPOCH as u64 * 3;
+        let root = MINIMUM_SLOTS_PER_EPOCH as u64 * 3;
         let shreds =
-            local_entries_to_shred(&[Entry::default()], slot + 1, slot - 1, &leader_keypair);
+            local_entries_to_shred(&[Entry::default()], root + 1, root - 1, &leader_keypair);
         assert!(!should_retransmit_and_persist(
             &shreds[0],
-            Some(bank),
+            Some(bank.clone()),
             &cache,
             &me_id,
-            slot,
+            root,
             0
         ));
 
-        // if the shred came back from me, it doesn't continue, whether or not I have a bank
+        // coding shreds don't contain parent slot information, test that slot >= root
+        let (common, coding) = Shredder::new_coding_shred_header(
+            5, // slot
+            5, // index
+            5, // fec_set_index
+            6, // num_data_shreds
+            6, // num_coding_shreds
+            3, // position
+            0, // version
+        );
+        let mut coding_shred =
+            Shred::new_empty_from_header(common, DataShredHeader::default(), coding);
+        Shredder::sign_shred(&leader_keypair, &mut coding_shred);
+        // shred.slot() > root, shred continues
+        assert!(should_retransmit_and_persist(
+            &coding_shred,
+            Some(bank.clone()),
+            &cache,
+            &me_id,
+            0,
+            0
+        ));
+        // shred.slot() == root, shred continues
+        assert!(should_retransmit_and_persist(
+            &coding_shred,
+            Some(bank.clone()),
+            &cache,
+            &me_id,
+            5,
+            0
+        ));
+        // shred.slot() < root, shred gets thrown out
         assert!(!should_retransmit_and_persist(
-            &shreds[0], None, &cache, &me_id, 0, 0
+            &coding_shred,
+            Some(bank),
+            &cache,
+            &me_id,
+            6,
+            0
         ));
     }
 
@@ -921,7 +958,7 @@ mod test {
     #[test]
     fn test_prune_shreds() {
         use {
-            crate::serve_repair::RepairType,
+            crate::serve_repair::ShredRepairType,
             std::net::{IpAddr, Ipv4Addr},
         };
         solana_logger::setup();
@@ -941,8 +978,8 @@ mod test {
             _from_addr,
             nonce: 0,
         };
-        let outstanding_requests = Arc::new(RwLock::new(OutstandingRepairs::default()));
-        let repair_type = RepairType::Orphan(9);
+        let outstanding_requests = Arc::new(RwLock::new(OutstandingShredRepairs::default()));
+        let repair_type = ShredRepairType::Orphan(9);
         let nonce = outstanding_requests
             .write()
             .unwrap()
